# Generated by Django 3.2.5 on 2021-07-27 15:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0024_alter_assessment_description'),
    ]

    operations = [
        migrations.AddField(
            model_name='assessment',
            name='open_to_start',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-25 11:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0021_alter_candidateassessment_assessment'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='job',
            name='assessment',
        ),
        migrations.AddField(
            model_name='job',
            name='assessments',
            field=models.ManyToManyField(to='job_board.Assessment'),
        ),
    ]



from django.contrib import admin
from django.template.defaultfilters import truncatechars_html
from django.urls import reverse
from django.utils.html import format_html, strip_tags
from django.utils.safestring import mark_safe

from job_board.models import AssessmentAnswer, AssessmentQuestion, Assessment


class AssessmentAnswerInline(admin.TabularInline):
    model = AssessmentAnswer
    fk_name = 'assessment_question'

    # readonly_fields = ['score']
    # min_num = 2

    def get_extra(self, request, obj=None, **kwargs):
        return 2 if not obj else 0


@admin.register(Assessment)
class AssessmentAdmin(admin.ModelAdmin):
    list_display = ('title', 'score', 'duration_display', 'get_description', 'type', 'open_to_start', 'show_action')

    @admin.display(description='description')
    def get_description(self, obj):
        return strip_tags(obj.description)

    @admin.display(description='Actions')
    def show_action(self, obj):
        return format_html(
            f'<a href="{reverse("preview_assessment", kwargs={"pk": obj.pk})}">Preview</a>'
        )


@admin.register(AssessmentQuestion)
class AssessmentQuestionAdmin(admin.ModelAdmin):
    list_display = ('assessment', 'get_title', 'score', 'type')
    list_filter = ('assessment',)
    inlines = (AssessmentAnswerInline,)

    ordering = ['assessment']

    change_form_template = 'admin/assessment_question/form.html'

    @admin.display(description='Title')
    def get_title(self, obj):
        safe_value = strip_tags(truncatechars_html(obj.title, 200))
        return mark_safe("&nbsp;".join(safe_value.split(' ')))

    def save_formset(self, request, form, formset, change):
        instances = formset.save(commit=False)
        correct_answer = self._correct_answer(formset)
        for obj in formset.deleted_objects:
            obj.delete()
        for instance in instances:
            if request.POST['type'] == 'single_choice':
                instance.score = 0
                if instance.correct:
                    instance.score = float(request.POST['score']) / correct_answer
            else:
                instance.score = - (float(request.POST['score']) / correct_answer)
                if instance.correct:
                    instance.score = float(request.POST['score']) / correct_answer
            instance.save()
        formset.save_m2m()

    def _correct_answer(self, formset):
        correct_answer = 0
        for form in formset:
            if form.cleaned_data['DELETE'] is False and form.cleaned_data['correct'] is True:
                correct_answer += 1
        return correct_answer



from rest_framework import serializers
from rest_framework.serializers import ModelSerializer

from job_board.models import Candidate, CandidateJob
from job_board.serializers.job_serializer import JobSerializerSimple


class CandidateSerializer(ModelSerializer):
    """
    Candidate serializer has been used in registration and fetch candidate
    this serializer is based on Candidate Model
    """

    class Meta:
        model = Candidate
        fields = ('id', 'full_name', 'email', 'password', 'phone', 'avatar', 'cv')
        extra_kwargs = {'password': {"write_only": True}}

    def get_cv_url(self, candidate):
        request = self.context.get('request')
        cv_url = candidate.cv.url
        return request.build_absolute_uri(cv_url)


class CandidateUpdateSerializer(serializers.ModelSerializer):
    """
    Candidate profile update serializer, this serializer has been used in candidate profile update
    """
    current_password = serializers.CharField()

    class Meta:
        model = Candidate
        fields = ('full_name', 'cv', 'avatar', 'current_password')
        extra_kwargs = {'cv': {'required': False}}


class CandidateJobApplySerializer(serializers.Serializer):
    job_slug = serializers.CharField()
    expected_salary = serializers.FloatField()
    additional_message = serializers.CharField(allow_null=True)

    def create(self, validated_data):
        validated_data.pop('job_slug')
        candidate_job = CandidateJob(**validated_data)
        candidate_job.save()
        return candidate_job

    def update(self, instance, validated_data):
        pass


class CandidateJobSerializer(serializers.ModelSerializer):
    job = JobSerializerSimple(many=False)
    created_at = serializers.DateTimeField(format='%d %B, %Y', read_only=True, required=False)

    class Meta:
        model = CandidateJob
        fields = ('unique_id', 'job', 'expected_salary', 'additional_message', 'created_at')



from rest_framework import serializers
from job_board.models import JobSummery, Job, JobAdditionalField


class JobSummerySerializer(serializers.ModelSerializer):
    application_deadline = serializers.DateField(format="%B %d %Y")
    job_type = serializers.CharField(source='get_job_type_display')

    class Meta:
        model = JobSummery
        fields = ['application_deadline', 'experience', 'job_type', 'vacancy', 'salary_range']


class AdditionalFieldsSerializer(serializers.ModelSerializer):
    class Meta:
        model = JobAdditionalField
        fields = ['title', 'required', 'validation_regx']


class JobSerializer(serializers.ModelSerializer):
    job_summery = JobSummerySerializer(many=False)
    additional_fields = AdditionalFieldsSerializer(many=True)
    updated_at = serializers.DateTimeField(format="%B %d %Y")

    # assessment = AssessmentSerializer(many=False)

    class Meta:
        model = Job
        fields = ['title', 'slug', 'job_context', 'job_description', 'job_responsibility',
                  'educational_requirement', 'additional_requirement', 'compensation', 'assessments',
                  'updated_at', 'job_summery', 'additional_fields']


class JobSerializerSimple(serializers.ModelSerializer):
    # assessment = AssessmentSerializer(many=False)

    class Meta:
        model = Job
        fields = ['title']



# Generated by Django 3.2.5 on 2021-07-14 10:37

from django.db import migrations, models
import job_board.models


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0006_auto_20210714_1333'),
    ]

    operations = [
        migrations.AddField(
            model_name='candidatejob',
            name='additional_message',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='candidatejob',
            name='expected_salary',
            field=models.FloatField(default=0.0),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='candidate',
            name='cv',
            field=models.FileField(upload_to=job_board.models.candidate_email_path),
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-28 13:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0027_auto_20210728_1546'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='assessment',
            name='score',
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-25 10:54

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0020_remove_candidateassessment_exam_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='candidateassessment',
            name='assessment',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to='job_board.assessment'),
            preserve_default=False,
        ),
    ]



from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html

from job_board.models import Candidate, CandidateJob, ResetPassword, CandidateAssessment


@admin.register(Candidate)
class CandidateAdmin(admin.ModelAdmin):
    change_form_template = 'admin/candidate/custom_candidate_form.html'

    list_display = ('full_name', 'email', 'phone', 'applied_job')

    def applied_job(self, obj: Candidate):
        return obj.candidatejob_set.count()

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = extra_context or {}
        extra_context['candidate_jobs'] = CandidateJob.objects.filter(candidate_id=object_id).all()
        return super(CandidateAdmin, self).change_view(request, object_id, form_url, extra_context=extra_context)


@admin.register(CandidateJob)
class CandidateJobAdmin(admin.ModelAdmin):
    list_display = ('candidate', 'get_job', 'get_assessment', 'unique_id')
    list_display_links = ('get_job', 'candidate')

    @admin.display(description='Job', ordering='job')
    def get_job(self, obj):
        return obj.job.title

    @admin.display(description='Assessment', ordering='job__assessment')
    def get_assessment(self, obj: CandidateJob):
        # url = reverse(f'admin:{obj.job.assessment._meta.app_label}_{obj.job.assessment._meta.model_name}_change',
        #               args=[obj.job.assessment.id])
        # return format_html(
        #     f'<a href="{url}">{obj.job.assessment}</a>'
        # )
        return 'ok'


@admin.register(CandidateAssessment)
class CandidateAssessment(admin.ModelAdmin):
    list_display = ('candidate', 'exam_started_at', 'get_assessment', 'score', 'status', 'result',)
    search_fields = ('score', 'candidate_job__candidate__full_name', 'candidate_job__candidate__email')
    list_filter = ('assessment', 'assessment__type', 'candidate_job__job__title')
    readonly_fields = ['step']

    def candidate(self, obj):
        return format_html(
            f'{obj.candidate_job.candidate.full_name}'
        )

    def get_assessment(self, obj):
        return format_html(
            f'{obj.assessment.title} </br>'
            f'Total Score : {obj.assessment.score} </br>'
            f'Pass Score : {obj.assessment.pass_score} </br>'
            f'{obj.assessment.get_type_display()}'
        )

    def assessment_pass_score(self, obj):
        return obj.assessment.pass_score


@admin.register(ResetPassword)
class CandidateResetPasswordAdmin(admin.ModelAdmin):
    list_display = ('email', 'otp', 'otp_expire_at', 'otp_used_at')
    readonly_fields = ('otp', 'otp_expire_at', 'otp_used_at')



# Generated by Django 3.2.5 on 2021-07-24 17:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0016_auto_20210724_2250'),
    ]

    operations = [
        migrations.AddField(
            model_name='candidateassessment',
            name='evaluation_url',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='candidateassessment',
            name='note',
            field=models.TextField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-25 10:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0018_auto_20210724_2332'),
    ]

    operations = [
        migrations.AddField(
            model_name='assessment',
            name='type',
            field=models.CharField(choices=[('mcq', 'MCQ Examination'), ('written', 'Written Examination'), ('viva', 'Viva/Oral Examination')], default='mcq', max_length=40),
        ),
        migrations.AlterField(
            model_name='candidateassessment',
            name='candidate_job',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='candidate_job', to='job_board.candidatejob'),
        ),
    ]



from datetime import timedelta

from django.core.exceptions import BadRequest
from django.utils import timezone
from rest_framework.generics import GenericAPIView
from rest_framework import mixins, viewsets, status, serializers
from rest_framework.response import Response
from rest_framework.views import APIView

from job_board.auth.CandidateAuth import CandidateAuth
from job_board.models import CandidateAssessment
from job_board.serializers.assessment_serializer import GivenAssessmentAnswerSerializer, \
    CandidateAssessmentSerializer, AssessmentQuestionSerializer


class CandidateAssessmentBase(GenericAPIView):
    """
    This is base class for , Candidate Assessment List, Candidate Assessment Retrieve, And Start Exam
    """
    serializer_class = CandidateAssessmentSerializer
    authentication_classes = [CandidateAuth]
    lookup_field = 'unique_id'

    def get_queryset(self):
        user = self.request.user
        return CandidateAssessment.objects.filter(candidate_job__candidate=user).all()


class CandidateAssessmentList(CandidateAssessmentBase, mixins.ListModelMixin):
    """
    This class will return assessment list by authenticated user,
    You will need postman to submit with authorization header.

    To collect jwt auth token, visit [login](/api/login)
    """

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)


class CandidateAssessmentView(CandidateAssessmentBase, mixins.RetrieveModelMixin, mixins.UpdateModelMixin):
    """
    Candidate assessment view support get and put method
    get method just retrieve the assessment by given unique_id (which is a uuid4)

    put method is responsible to start the exam with following conditions
    if the exam is open_to_start and it's does not started yet
    if the exam is not open_to_start and candidate_assessment can_start_after is less then current date
    """

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)

    def put(self, request, *args, **kwargs):
        candidate_assessment = self.get_object()
        if candidate_assessment.assessment.open_to_start and candidate_assessment.exam_started_at is None:
            self._start_exam(candidate_assessment=candidate_assessment)
            return Response({'message': 'Exam started'})
        if candidate_assessment.can_start_after and \
                candidate_assessment.can_start_after <= timezone.now() and \
                candidate_assessment.exam_started_at is None:
            self._start_exam(candidate_assessment=candidate_assessment)
            return Response({'message': 'Exam Started'})

        msg = f'This assessment is not open to start, you can start exam after {candidate_assessment.can_start_after}'
        return Response({'admin_only': msg}, status=status.HTTP_403_FORBIDDEN)

    def _start_exam(self, candidate_assessment):
        candidate_assessment.exam_started_at = timezone.now()
        candidate_assessment.exam_end_at = timezone.now() + timedelta(
            minutes=candidate_assessment.assessment.duration)
        candidate_assessment.step = {
            'current_step': 0,
            'question_ids': list(
                candidate_assessment.assessment.assessmentquestion_set.values_list('id', flat=True))
        }
        candidate_assessment.save()


class CandidateAssessmentQuestion(APIView):
    """
    Candidate Assessment Question only support get method
    it will lookup for candidate assessment by uuid and if the exam has started already
    it will return question serializer if have left step
    it will return time_up error if exam time up
    """

    def get(self, request, *args, **kwargs):
        """

        @param request:
        @param args:
        @param kwargs:
        @return:
        """
        kwargs['exam_started_at__isnull'] = False
        try:
            candidate_assessment = CandidateAssessment.objects.filter(**kwargs).first()
            question_serializer = AssessmentQuestionSerializer(self.get_question(candidate_assessment))
            if not candidate_assessment.time_spend == 'time_up':
                return Response(question_serializer.data)
            return Response({'time_up': 'You have spend all the minutes ! best of luck'},
                            status=status.HTTP_400_BAD_REQUEST)
        except BadRequest:
            return Response({'message': 'Bad Request'}, status=status.HTTP_400_BAD_REQUEST)

    def get_question(self, candidate_assessment: CandidateAssessment):
        try:
            question_id = candidate_assessment.step['question_ids'][candidate_assessment.step['current_step']]
            return candidate_assessment.assessment.assessmentquestion_set.filter(id=question_id).first()
        except:
            raise serializers.ValidationError(
                {'out_of_step': f'You have answered all the question we have with {candidate_assessment.assessment}'})


class SaveAnswerView(GenericAPIView, mixins.CreateModelMixin):
    """
    Save answer question
    """
    serializer_class = GivenAssessmentAnswerSerializer
    authentication_classes = [CandidateAuth]

    def post(self, request, *args, **kwargs):
        """
        save answer and return next question
        @param request:
        @param args:
        @param kwargs:
        @return:
        """
        self.create(request, *args, **kwargs)
        return Response({'d': 'd'})



from django.shortcuts import redirect
from django.urls import path, include
from rest_framework.urlpatterns import format_suffix_patterns

from job_board.apis import job, authentication, assessment
from job_board.views import AssessmentPreview, WebsiteView

urlpatterns = [
    path('api/', include([
        path('register-candidate/', authentication.Registration.as_view(), name='jb_registration'),
        path('login/', authentication.Login.as_view(), name='jb_login'),
        path('candidate/', authentication.User.as_view(), name='jb_candidate'),
        path('send-otp/', authentication.SendOTP.as_view(), name='jb_send_otp'),
        path('reset-password/', authentication.ResetPasswordView.as_view(), name='jb_reset_password'),
        path('jobs/', job.JobList.as_view(), name='jb_jobs'),
        path('job/<str:slug>/', job.JobRetrieve.as_view(), name='jb_job'),
        path('apply/', job.CandidateJobView.as_view(), name='jb_job_apply'),
        path('assessment/', assessment.CandidateAssessmentList.as_view(), name='jb_assessments'),
        path('assessment/save-answer/', assessment.SaveAnswerView.as_view(), name='jb_save_answer'),  # POST
        path('assessment/<str:unique_id>/', assessment.CandidateAssessmentView.as_view(), name='jb_assessment'),
        # GET, POST
        path('assessment/<str:unique_id>/question/', assessment.CandidateAssessmentQuestion.as_view(),
             name='fetch_question'),  # GET

    ])),
    path('admin/job-board/assessment/<int:pk>/preview/', AssessmentPreview.as_view(),
         name='preview_assessment'),
    # path('job-board/', WebsiteView.as_view())
]

urlpatterns = format_suffix_patterns(urlpatterns)



# Generated by Django 3.2.5 on 2021-07-14 07:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0005_auto_20210702_1224'),
    ]

    operations = [
        migrations.AddField(
            model_name='candidate',
            name='email_otp',
            field=models.CharField(blank=True, max_length=10, null=True),
        ),
        migrations.AddField(
            model_name='candidate',
            name='email_verified_at',
            field=models.DateField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.2.4 on 2021-06-29 10:00

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0002_job_assessment'),
    ]

    operations = [
        migrations.AlterField(
            model_name='job',
            name='assessment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='job_board.assessment'),
        ),
    ]



from django.contrib.admin import AdminSite
from django.contrib.auth.mixins import LoginRequiredMixin, AccessMixin
from django.core.exceptions import PermissionDenied
from django.views.generic import TemplateView
from job_board.models import Assessment


class AssessmentPreview(LoginRequiredMixin, AccessMixin, AdminSite, TemplateView):
    template_name = 'admin/assessment/preview.html'

    def get_context_data(self, **kwargs):
        assessment = Assessment.objects.filter(**kwargs).first()
        # print(self.get_app_list())
        return {
            'assessment': assessment,
            'site_header': 'Mediusware Ltd',
            'opts': Assessment._meta,
            'is_nav_sidebar_enabled': False,
            'available_apps': [],
            'has_permission': True
        }

    def dispatch(self, request, *args, **kwargs):
        if request.user.has_perm('preview_assessment'):
            return super(AssessmentPreview, self).dispatch(request, *args, **kwargs)
        raise PermissionDenied("Permission denied")


class WebsiteView(TemplateView):
    template_name = 'website/index.html'



# Generated by Django 3.2.5 on 2021-07-28 16:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('job_board', '0030_candidateassessment_can_open_after'),
    ]

    operations = [
        migrations.RenameField(
            model_name='candidateassessment',
            old_name='can_open_after',
            new_name='can_start_after',
        ),
    ]



from functools import wraps


def admin_required(view_func, arg):
    def decorated_function(request, *args, **kwargs):
        print(request)
        print(arg)
        return view_func(request, *args, **kwargs)

    return decorated_function



from django.core.mail import EmailMultiAlternatives


def send_otp(otp, email_address):
    email = EmailMultiAlternatives(subject='Mediusware Job - Password Reset')
    html_content = f'<h1>Mediusware Ltd</h1> ' \
                   f'<p>Use the following OTP to complete your Sign Up procedures. OTP is valid for 5 minutes</p>' \
                   f'<br>' \
                   f'{otp}' \
                   f'<hr>' \
                   f'<br>' \
                   f'<br>' \
                   f'Regards,' \
                   f'Mediusware Job'
    email.attach_alternative(html_content, 'text/html')
    email.to = [email_address]
    email.send()
